//  2007 Alessandro Marianantoni <alex@alexrieti.com>
 
// MessageListenerService.java
// It connects to the reader and it receives UDP packets 
 

 /*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; version 2.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

 */

package driver.easyreader;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import driver.easyreader.Sputnik;

public class MessageListenerService implements Runnable {

	InetAddress server;

	DatagramSocket sock;

	DatagramPacket packet;

	int c = 0;

	Thread t;

	MessageListener iMessageListener;

	boolean debug = false;

	boolean running = false;

	int[] tea_key;

	Sputnik Sputnik;

	public MessageListenerService(String host, int port, int timeout, int[] key)
			throws InterruptedException {

		boolean bound = false;
		int attempts = 0;
		tea_key = key;

		while (!(bound || (attempts >= 3))) {
			try {
				server = InetAddress.getByName(host);
				sock = new DatagramSocket(port, server);
				bound = sock.isBound();
			} catch (IOException ioe) {
				System.err.println("MessageListenerService: " + (attempts + 1)
						+ " attempt to connect to: " + host + " IOexc:"
						+ ioe.toString());
				attempts++;
				Thread.sleep(5000);
			}
		}
		if (!bound) {
			System.out
					.println("MessageListenerService: Unable to connect, exit.");
			Runtime.getRuntime().exit(1);
		} else {
			System.out.println("Socket bound");
		}

		try {
			sock.setSoTimeout(timeout * 1000);
			sock.setReceiveBufferSize(1);
		} catch (IOException ioe) {
			System.err.println("Error Setup Socket MessageListenerService: "
					+ ioe.toString());
		}

		t = new Thread(this);

		if (debug) {
			try {
				System.out.println("Local IP: "
						+ sock.getLocalAddress().getHostAddress() + ":"
						+ sock.getLocalPort() + "  receive buffer size: "
						+ sock.getReceiveBufferSize() + "  timeout: "
						+ sock.getSoTimeout());
			} catch (IOException ioe) {
				System.err
						.println("Error printing info MessageListenerService: "
								+ ioe.toString());
			}
		}

		packet = new DatagramPacket(new byte[256], 256);
	}

	public void setDebug(boolean de) {
		debug = de;
	}

	public void Start() {
		running = true;
		t.start();
	}

	public boolean isRunning() {
		return running;
	}

	public void setMessageListener(MessageListener O) {
		iMessageListener = O;
	}

	public void run() {
		while (isRunning()) {
			try {
				sock.receive(packet);
				Sputnik = new Sputnik(packet, tea_key);
				if (debug) {
					System.out.print(c + " | lenght:" + Sputnik.getSize()
							+ " | sent by: " + Sputnik.getBeaconAddress()
							+ " | SputnikID: " + Sputnik.getID());
				}
				if (Sputnik.checkCRC()) {
					iMessageListener.messageReceived(Sputnik);
					c++;
				} else
					System.out.println("Packet lost  CRC:" + Sputnik.getCRC());
			} catch (IOException e) {
				// there is probably a better way..
				if (e.toString().equalsIgnoreCase(
						"java.net.SocketTimeoutException: Receive timed out"))
					System.out.println("No Tags");
			}

		}

	}

}
