//  2007 Alessandro Marianantoni <alex@alexrieti.com>
 
// Sputnik.java
// Software object for the Sputnik HW module 
 

 /*
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; version 2.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/*

 in order this represents the 16BYTE received from the EasyOpenBeacon:
 Size_1B
 Proto_1B
 Flags_1B
 Strength_1B
 Seq_4B
 TagID_4B
 reserved_2B
 crc_2B
 
 */

package driver.easyreader;

import java.net.DatagramPacket;
import java.net.InetAddress;
import java.nio.ByteBuffer;

public class Sputnik {

	private InetAddress beaconAddress;

	private byte sArray[];

	public long time;

	public Sputnik(DatagramPacket packet, int[] tea_key) {
		int i;
		byte[] data16 = new byte[16];

		beaconAddress = packet.getAddress();
		ByteBuffer buf = ByteBuffer.wrap(packet.getData());

		time = System.currentTimeMillis();
		// obtain the 16Byte packet
		for (i = 0; i < packet.getLength(); i++)
			data16[i] = buf.get();

		if (tea_key != null) {

			// flip encrypted array from network byte order to host
			byte flip_array[] = flipArray(data16);

			// all packet decryption
			byte[] decrypt = XXtea.decrypt(flip_array, tea_key);

			// flip decrypted data from network byte order to host
			sArray = flipArray(decrypt);
		} else {
			sArray = data16;
		}
	}

	public int getID() {
		return ((0xff & sArray[8]) << 24) + ((0xff & sArray[9]) << 16)
				+ ((0xff & sArray[10]) << 8) + ((0xff & sArray[11]));
	}

	public InetAddress getBeaconAddress() {
		return beaconAddress;
	}

	public int getStrength() {
		return (0xff & sArray[3]);
	}

	public int getSize() {
		return (0xff & sArray[0]);
	}

	public int getProto() {
		return (0xff & sArray[1]);
	}

	public int getFlag() {
		return (0xff & sArray[2]);
	}

	public int getSeq() {
		return ((0xff & sArray[4]) << 24) + ((0xff & sArray[5]) << 16)
				+ ((0xff & sArray[6]) << 8) + ((0xff & sArray[7]));
	}

	public int getReserved() {
		return ((0xff & sArray[12]) + (0xff & sArray[13]));
	}

	public int getCRC() {
		return ((0xff & sArray[14]) << 8) + ((0xff & sArray[15]));
	}

	public boolean checkCRC() {
		int size = 14;
		long crc = 0xFFFF;
		int p = 0;
		while (size-- > 0) {
			crc = 0xFFFF & ((crc >> 8) | (crc << 8));
			crc ^= 0xFF & sArray[p++];
			crc ^= ((0xff & crc) >> 4) & 0xFFFF;
			crc ^= (crc << 12) & 0xFFFF;
			crc ^= ((crc & 0xFF) << 5) & 0xFFFF;
		}
		crc = (0xFFFF & crc);
		return (crc == getCRC());
	}

	public long getTime() {
		return time;
	}

	private byte[] flipArray(byte[] input) {

		int INT_SIZE = 4;
		int i, j;
		byte flipped_array[] = new byte[16];

		for (i = 0; i < 16 / INT_SIZE; i++) {
			for (j = 0; j < INT_SIZE; j++) {
				flipped_array[(i + 1) * INT_SIZE - j - 1] = input[i * INT_SIZE
						+ j];
			}
		}
		return flipped_array;
	}
}
